# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
import base64
from datetime import datetime
from datetime import timedelta

from tempest import config
from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base as rbac_base

CONF = config.CONF


class BarbicanV1RbacSecrets(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def test_create_secret(self):
        """Test add_secret policy.

        Testing: POST /v1/secrets
        This test must check:
          * whether the persona can create an empty secret
          * whether the persona can create a secret with a symmetric key
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_list_secrets(self):
        """Test get_secrets policy.

        Testing: GET /v1/secrets
        This test must check:
          * whether the persona can list secrets within their project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_secret(self):
        """Test deleting a secret.

        Testing: DEL /v1/secrets/{secret_id}
        This test must check:
          * whether the persona can delete a secret in their project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret(self):
        """Test get_secret policy.

        Testing: GET /v1/secrets/{secret_id}
        This test must check:
          * whether the persona can get a specific secret within their project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret_payload(self):
        """Test get_secret payload policy.

        Testing: GET /v1/secrets/{secret_id}/payload
        This test must check:
          * whether the persona can get a secret payload
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_put_secret_payload(self):
        """Test put_secret policy.

        Testing: PUT /v1/secrets/{secret_id}
        This test must check:
          * whether the persona can add a paylod to an empty secret
        """
        raise NotImplementedError


class ProjectMemberTests(rbac_base.BarbicanV1RbacBase, BarbicanV1RbacSecrets):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.secret_v1.SecretClient()

    def test_create_secret(self):
        """Test add_secret policy."""
        self.do_request('create_secret', expected_status=201, cleanup='secret',
                        name='test_create_secret')

        key = rbac_base.create_aes_key()
        expire_time = (datetime.utcnow() + timedelta(days=5))
        self.do_request(
            'create_secret', expected_status=201, cleanup="secret",
            name='test_create_secret2',
            expiration=expire_time.isoformat(), algorithm="aes",
            bit_length=256, mode="cbc", payload=key,
            payload_content_type="application/octet-stream",
            payload_content_encoding="base64"
        )

    def test_list_secrets(self):
        """Test get_secrets policy."""
        # create two secrets
        self.create_empty_secret_admin('test_list_secrets')
        self.create_empty_secret_admin('test_list_secrets_2')

        # list secrets with name secret_1
        resp = self.do_request('list_secrets', name='test_list_secrets')
        secrets = resp['secrets']
        self.assertEqual('test_list_secrets', secrets[0]['name'])

        # list secrets with name secret_2
        resp = self.do_request('list_secrets', name='test_list_secrets_2')
        secrets = resp['secrets']
        self.assertEqual('test_list_secrets_2', secrets[0]['name'])

        # list all secrets
        resp = self.do_request('list_secrets')
        secrets = resp['secrets']
        self.assertGreaterEqual(len(secrets), 2)

    def test_delete_secret(self):
        """Test delete_secrets policy."""
        sec = self.create_empty_secret_admin('test_delete_secret_1')
        uuid = self.ref_to_uuid(sec['secret_ref'])
        self.do_request('delete_secret', secret_id=uuid)
        self.delete_cleanup('secret', uuid)

    def test_get_secret(self):
        """Test get_secret policy."""
        sec = self.create_empty_secret_admin('test_get_secret')
        uuid = self.ref_to_uuid(sec['secret_ref'])
        resp = self.do_request('get_secret_metadata', secret_id=uuid)
        self.assertEqual(uuid, self.ref_to_uuid(resp['secret_ref']))

    def test_get_secret_payload(self):
        """Test get_secret payload policy."""
        key, sec = self.create_aes_secret_admin('test_get_secret_payload')
        uuid = self.ref_to_uuid(sec['secret_ref'])

        # Retrieve the payload
        payload = self.do_request('get_secret_payload', secret_id=uuid)
        self.assertEqual(key, base64.b64encode(payload))

    def test_put_secret_payload(self):
        """Test put_secret policy."""
        sec = self.create_empty_secret_admin('test_put_secret_payload')
        uuid = self.ref_to_uuid(sec['secret_ref'])

        key = rbac_base.create_aes_key()

        # Associate the payload with the created secret
        self.do_request('put_secret_payload', secret_id=uuid, payload=key)

        # Retrieve the payload
        payload = self.do_request('get_secret_payload', secret_id=uuid)
        self.assertEqual(key, base64.b64encode(payload))


class ProjectAdminTests(ProjectMemberTests):
    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.secret_v1.SecretClient()


class ProjectReaderTests(rbac_base.BarbicanV1RbacBase, BarbicanV1RbacSecrets):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.SecretClient()

    def test_create_secret(self):
        """Test add_secret policy."""
        self.do_request(
            'create_secret', expected_status=exceptions.Forbidden,
            cleanup='secret')

        key = rbac_base.create_aes_key()
        expire_time = (datetime.utcnow() + timedelta(days=5))
        self.do_request(
            'create_secret', expected_status=exceptions.Forbidden,
            cleanup="secret",
            expiration=expire_time.isoformat(), algorithm="aes",
            bit_length=256, mode="cbc", payload=key,
            payload_content_type="application/octet-stream",
            payload_content_encoding="base64"
        )

    def test_list_secrets(self):
        """Test get_secrets policy."""
        # create two secrets
        self.create_empty_secret_admin('secret_1')
        self.create_empty_secret_admin('secret_2')

        # list secrets with name secret_1
        self.do_request(
            'list_secrets', expected_status=exceptions.Forbidden,
            name='secret_1'
        )

        # list secrets with name secret_2
        self.do_request(
            'list_secrets', expected_status=exceptions.Forbidden,
            name='secret_2'
        )

        # list all secrets
        self.do_request(
            'list_secrets', expected_status=exceptions.Forbidden
        )

    def test_delete_secret(self):
        """Test delete_secrets policy."""
        sec = self.create_empty_secret_admin('secret_1')
        uuid = self.ref_to_uuid(sec['secret_ref'])
        self.do_request(
            'delete_secret', expected_status=exceptions.Forbidden,
            secret_id=uuid
        )

    def test_get_secret(self):
        """Test get_secret policy."""
        sec = self.create_empty_secret_admin('secret_1')
        uuid = self.ref_to_uuid(sec['secret_ref'])
        self.do_request(
            'get_secret_metadata', expected_status=exceptions.Forbidden,
            secret_id=uuid
        )

    def test_get_secret_payload(self):
        """Test get_secret payload policy."""
        key, sec = self.create_aes_secret_admin('secret_1')
        uuid = self.ref_to_uuid(sec['secret_ref'])

        # Retrieve the payload
        self.do_request(
            'get_secret_payload', expected_status=exceptions.Forbidden,
            secret_id=uuid
        )

    def test_put_secret_payload(self):
        """Test put_secret policy."""
        sec = self.create_empty_secret_admin('secret_1')
        uuid = self.ref_to_uuid(sec['secret_ref'])

        key = rbac_base.create_aes_key()

        # Associate the payload with the created secret
        self.do_request(
            'put_secret_payload', expected_status=exceptions.Forbidden,
            secret_id=uuid, payload=key
        )


class SystemAdminTests(rbac_base.BarbicanV1RbacBase, BarbicanV1RbacSecrets):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.secret_client

    def test_create_secret(self):
        pass

    def test_list_secrets(self):
        pass

    def test_delete_secret(self):
        pass

    def test_get_secret(self):
        pass

    def test_get_secret_payload(self):
        pass

    def test_put_secret_payload(self):
        pass


class SystemMemberTests(rbac_base.BarbicanV1RbacBase, BarbicanV1RbacSecrets):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.secret_client

    def test_create_secret(self):
        pass

    def test_list_secrets(self):
        pass

    def test_delete_secret(self):
        pass

    def test_get_secret(self):
        pass

    def test_get_secret_payload(self):
        pass

    def test_put_secret_payload(self):
        pass


class SystemReaderTests(rbac_base.BarbicanV1RbacBase, BarbicanV1RbacSecrets):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.secret_client

    def test_create_secret(self):
        pass

    def test_list_secrets(self):
        pass

    def test_delete_secret(self):
        pass

    def test_get_secret(self):
        pass

    def test_get_secret_payload(self):
        pass

    def test_put_secret_payload(self):
        pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from tempest.lib.common import rest_client


_DEFAULT_SERVICE_TYPE = 'key-manager'


class BarbicanTempestClient(rest_client.RestClient):

    def __init__(self, *args, **kwargs):
        kwargs['service'] = _DEFAULT_SERVICE_TYPE
        super().__init__(*args, **kwargs)



# Copyright (c) 2016 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from barbican_tempest_plugin.tests.api import base


class ConsumersTest(base.BaseKeyManagerTest):
    """Containers API tests."""

    @decorators.idempotent_id('7d46a170-6b3b-4f4d-903a-b29aebb93289')
    def test_add_delete_consumers_in_container(self):
        # Create a container to test against
        body = self.create_container(
            type="generic",
            name="consumer-container"
        )
        container_id = base._get_uuid(body.get('container_ref'))

        # Confirm that the container has no consumers
        body = self.consumer_client.list_consumers_in_container(container_id)
        self.assertEqual(0, body.get('total'), body)
        self.assertEmpty(body.get('consumers'), body)

        # Add some consumers to the container
        body = self.consumer_client.add_consumer_to_container(
            container_id,
            name="consumer1",
            URL="url1"
        )
        self.assertEqual(
            container_id,
            base._get_uuid(body.get('container_ref')),
            body
        )
        self.assertEqual(1, len(body.get('consumers')), body)
        body = self.consumer_client.add_consumer_to_container(
            container_id,
            name="consumer2",
            URL="url2"
        )
        self.assertEqual(
            container_id,
            base._get_uuid(body.get('container_ref')),
            body
        )
        self.assertEqual(2, len(body.get('consumers')), body)

        # Confirm that the consumers are in the container
        body = self.consumer_client.list_consumers_in_container(container_id)
        self.assertEqual(2, body.get('total'), body)
        self.assertEqual(2, len(body.get('consumers')), body)
        for consumer in body.get('consumers'):
            self.assertIn(consumer.get('name'), ("consumer1", "consumer2"))
            self.assertIn(consumer.get('URL'), ("url1", "url2"))

        # Remove the consumers from the container
        body = self.consumer_client.delete_consumer_from_container(
            container_id,
            name="consumer1",
            URL="url1"
        )
        self.assertEqual(
            container_id,
            base._get_uuid(body.get('container_ref')),
            body
        )
        self.assertEqual(1, len(body.get('consumers')), body)
        body = self.consumer_client.delete_consumer_from_container(
            container_id,
            name="consumer2",
            URL="url2"
        )
        self.assertEqual(
            container_id,
            base._get_uuid(body.get('container_ref')),
            body
        )
        self.assertEqual(0, len(body.get('consumers')), body)

        # Confirm that the container has no consumers
        body = self.consumer_client.list_consumers_in_container(container_id)
        self.assertEqual(0, body.get('total'), body)
        self.assertEqual(0, len(body.get('consumers')), body)

        # Clean up the container
        self.delete_container(container_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import base64
from datetime import datetime
from datetime import timedelta
import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

from tempest import clients
from tempest import config
from tempest.lib import auth
from tempest.lib.common.utils import data_utils
from tempest import test

CONF = config.CONF

RESOURCE_TYPES = ['container', 'order', 'quota', 'secret']


def create_aes_key():
    password = b"password"
    salt = os.urandom(16)
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(), length=32, salt=salt,
        iterations=1000, backend=default_backend()
    )
    return base64.b64encode(kdf.derive(password))


class BarbicanV1RbacBase(test.BaseTestCase):

    identity_version = 'v3'
    _created_projects = None
    _created_users = None
    created_objects = {}

    credentials = ['system_admin']

    @classmethod
    def ref_to_uuid(cls, href):
        return href.split('/')[-1]

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not CONF.barbican_rbac_scope_verification.enforce_scope:
            raise cls.skipException("enforce_scope is not enabled for "
                                    "barbican, skipping RBAC tests")

    @classmethod
    def setup_credentials(cls):
        super().setup_credentials()
        cls._created_projects = list()
        cls._created_users = list()
        project_id = cls.os_system_admin.projects_client.create_project(
            data_utils.rand_name()
        )['project']['id']
        cls._created_projects.append(project_id)
        setattr(cls, 'os_project_admin',
                cls._setup_new_user_client(project_id, 'admin'))
        setattr(cls, 'os_project_member',
                cls._setup_new_user_client(project_id, 'member'))
        setattr(cls, 'os_project_reader',
                cls._setup_new_user_client(project_id, 'reader'))

    @classmethod
    def _setup_new_user_client(cls, project_id, role):
        """Create a new tempest.clients.Manager

        Creates a new user with the given roles on the given project,
        and returns an instance of tempest.clients.Manager set up
        for that user.

        Users are cleaned up during class teardown in cls.clear_credentials
        """
        user = {
            'name': data_utils.rand_name('user'),
            'password': data_utils.rand_password()
        }
        user_id = cls.os_system_admin.users_v3_client.create_user(
            **user
        )['user']['id']
        cls._created_users.append(user_id)
        role_id = cls.os_system_admin.roles_v3_client.list_roles(
            name=role
        )['roles'][0]['id']
        cls.os_system_admin.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id
        )
        creds = auth.KeystoneV3Credentials(
            user_id=user_id,
            password=user['password'],
            project_id=project_id
        )
        auth_provider = clients.get_auth_provider(creds)
        creds = auth_provider.fill_credentials()
        return clients.Manager(credentials=creds)

    @classmethod
    def clear_credentials(cls):
        for user_id in cls._created_users:
            cls.os_system_admin.users_v3_client.delete_user(user_id)
        for project_id in cls._created_projects:
            cls.os_system_admin.projects_client.delete_project(project_id)
        super().clear_credentials()

    @classmethod
    def setup_clients(cls):
        super().setup_clients()

        # setup clients for primary persona
        os = cls.os_project_member
        cls.secret_client = os.secret_v1.SecretClient()
        cls.secret_metadata_client = os.secret_v1.SecretMetadataClient(
            service='key-manager'
        )
        cls.consumer_client = os.secret_v1.ConsumerClient(
            service='key-manager'
        )
        cls.container_client = os.secret_v1.ContainerClient()
        cls.order_client = os.secret_v1.OrderClient()
        cls.quota_client = os.secret_v1.QuotaClient()
        cls.secret_client = os.secret_v1.SecretClient()
        cls.secret_metadata_client = os.secret_v1.SecretMetadataClient(
            service='key-manager'
        )

        # setup clients for admin persona
        # this client is used for any cleanupi/setup etc. as needed
        adm = cls.os_project_admin
        cls.admin_secret_client = adm.secret_v1.SecretClient()
        cls.admin_secret_metadata_client = adm.secret_v1.SecretMetadataClient(
            service='key-manager'
        )
        cls.admin_consumer_client = adm.secret_v1.ConsumerClient(
            service='key-manager'
        )
        cls.admin_container_client = adm.secret_v1.ContainerClient()
        cls.admin_order_client = adm.secret_v1.OrderClient()
        cls.admin_quota_client = adm.secret_v1.QuotaClient()
        cls.admin_secret_client = adm.secret_v1.SecretClient(
            service='key-manager'
        )
        cls.admin_secret_metadata_client = adm.secret_v1.SecretMetadataClient(
            service='key-manager'
        )

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        for resource in RESOURCE_TYPES:
            cls.created_objects[resource] = set()

    @classmethod
    def resource_cleanup(cls):
        try:
            for container_uuid in list(cls.created_objects['container']):
                cls.admin_container_client.delete_container(container_uuid)
                cls.created_objects['container'].remove(container_uuid)
            for order_uuid in list(cls.created_objects['order']):
                cls.admin_order_client.delete_order(order_uuid)
                cls.created_objects['order'].remove(order_uuid)
            for quota_uuid in list(cls.created_objects['quota']):
                cls.admin_quota_client.delete_project_quota(quota_uuid)
                cls.created_objects['quota'].remove(quota_uuid)
            for secret_uuid in list(cls.created_objects['secret']):
                cls.admin_secret_client.delete_secret(secret_uuid)
                cls.created_objects['secret'].remove(secret_uuid)
        finally:
            super(BarbicanV1RbacBase, cls).resource_cleanup()

    @classmethod
    def add_cleanup(cls, resource, response):
        if resource == 'container':
            uuid = cls.ref_to_uuid(response['container_ref'])
        if resource == 'order':
            uuid = cls.ref_to_uuid(response.get('order_ref'))
            order_metadata = cls.admin_order_client.get_order(uuid)
            secret_ref = order_metadata.get('secret_ref')
            if secret_ref:
                cls.created_objects['secret'].add(cls.ref_to_uuid(secret_ref))
            uuid = cls.ref_to_uuid(response['order_ref'])
        if resource == 'quota':
            uuid = cls.ref_to_uuid(response['quota_ref'])
        if resource == 'secret':
            uuid = cls.ref_to_uuid(response['secret_ref'])
        cls.created_objects[resource].add(uuid)

    @classmethod
    def delete_cleanup(cls, resource, uuid):
        cls.created_objects[resource].remove(uuid)

    def do_request(self, method, client=None, expected_status=200,
                   cleanup=None, **args):
        if client is None:
            client = self.client
        if isinstance(expected_status, type(Exception)):
            self.assertRaises(expected_status,
                              getattr(client, method),
                              **args)
        else:
            response = getattr(client, method)(**args)
            # self.assertEqual(response.response.status, expected_status)
            if cleanup is not None:
                self.add_cleanup(cleanup, response)
            return response

    def create_empty_secret_admin(self, secret_name):
        """add empty secret as admin user """
        return self.do_request(
            'create_secret', client=self.admin_secret_client,
            expected_status=201, cleanup='secret', name=secret_name)

    def create_aes_secret_admin(self, secret_name):
        key = create_aes_key()
        expire_time = (datetime.utcnow() + timedelta(days=5))
        return key, self.do_request(
            'create_secret', client=self.admin_secret_client,
            expected_status=201, cleanup="secret",
            expiration=expire_time.isoformat(), algorithm="aes",
            bit_length=256, mode="cbc", payload=key,
            payload_content_type="application/octet-stream",
            payload_content_encoding="base64",
            name=secret_name
        )



# Copyright 2016 SAP SE
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools

from tempest import config
from tempest import test

from barbican_tempest_plugin import clients

CONF = config.CONF

# NOTE(dane-fichter): We need to track resource types for cleanup.
RESOURCE_TYPES = ['container', 'order', 'quota', 'secret']


def _get_uuid(href):
    return href.split('/')[-1]


def creates(resource):
    """Decorator that adds resource UUIDs to queue for cleanup"""

    def decorator(f):
        @functools.wraps(f)
        def wrapper(cls, *args, **kwargs):
            resp = f(cls, *args, **kwargs)
            if resource == 'container':
                uuid = _get_uuid(resp['container_ref'])
            if resource == 'order':
                uuid = _get_uuid(resp.get('order_ref'))
                order_metadata = cls.get_order(uuid)
                secret_ref = order_metadata.get('secret_ref')
                if secret_ref:
                    cls.created_objects['secret'].add(_get_uuid(secret_ref))
                uuid = _get_uuid(resp['order_ref'])
            if resource == 'quota':
                uuid = _get_uuid(args[0])
            if resource == 'secret':
                uuid = _get_uuid(resp['secret_ref'])
            cls.created_objects[resource].add(uuid)
            return resp
        return wrapper
    return decorator


class BaseKeyManagerTest(test.BaseTestCase):
    """Base class for all api tests."""

    # Why do I have to be an admin to create secrets? No idea...
    credentials = ('admin', ['service_admin', 'key-manager:service-admin'])
    client_manager = clients.Clients
    created_objects = {}

    @classmethod
    def setup_clients(cls):
        super(BaseKeyManagerTest, cls).setup_clients()
        os = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.consumer_client = os.secret_v1.ConsumerClient(
            service='key-manager'
        )
        cls.container_client = os.secret_v1.ContainerClient(
            service='key-manager'
        )
        cls.order_client = os.secret_v1.OrderClient(service='key-manager')
        cls.secret_client = os.secret_v1.SecretClient(service='key-manager')
        cls.secret_metadata_client = os.secret_v1.SecretMetadataClient(
            service='key-manager'
        )

        os = getattr(cls, 'os_roles_%s' % cls.credentials[1][0])
        cls.quota_client = os.secret_v1.QuotaClient(service='key-manager')

    @classmethod
    def setup_credentials(cls):
        super().setup_credentials()
        cls.os_primary = getattr(cls, f'os_{cls.credentials[0]}')

    @classmethod
    def resource_setup(cls):
        super(BaseKeyManagerTest, cls).resource_setup()
        for resource in RESOURCE_TYPES:
            cls.created_objects[resource] = set()

    @classmethod
    def resource_cleanup(cls):
        try:
            for container_uuid in list(cls.created_objects['container']):
                cls.delete_container(container_uuid)
            for order_uuid in list(cls.created_objects['order']):
                cls.delete_order(order_uuid)
            for project_quota_uuid in list(cls.created_objects['quota']):
                cls.delete_project_quota(project_quota_uuid)
            for secret_uuid in list(cls.created_objects['secret']):
                cls.delete_secret(secret_uuid)
        finally:
            super(BaseKeyManagerTest, cls).resource_cleanup()

    @classmethod
    @creates('container')
    def create_container(cls, **kwargs):
        return cls.container_client.create_container(**kwargs)

    @classmethod
    def delete_container(cls, uuid):
        cls.created_objects['container'].remove(uuid)
        return cls.container_client.delete_container(uuid)

    @classmethod
    @creates('order')
    def create_order(cls, **kwargs):
        return cls.order_client.create_order(**kwargs)

    @classmethod
    def get_order(cls, uuid):
        return cls.order_client.get_order(uuid)

    @classmethod
    def delete_order(cls, uuid):
        cls.created_objects['order'].remove(uuid)
        return cls.order_client.delete_order(uuid)

    @classmethod
    @creates('quota')
    def create_project_quota(cls, project_id, **kwargs):
        return cls.quota_client.create_project_quota(project_id, **kwargs)

    @classmethod
    def delete_project_quota(cls, project_id):
        cls.created_objects['quota'].remove(project_id)
        return cls.quota_client.delete_project_quota(project_id)

    @classmethod
    @creates('secret')
    def create_secret(cls, **kwargs):
        return cls.secret_client.create_secret(**kwargs)

    @classmethod
    def delete_secret(cls, uuid):
        cls.created_objects['secret'].remove(uuid)
        return cls.secret_client.delete_secret(uuid)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import abc

from tempest import config
from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


CONF = config.CONF


class BarbicanV1RbacSecretStores:

    @abc.abstractmethod
    def test_list_secret_stores(self):
        """Test getting a list of all backends

        Testing: GET /v1/secret-stores
        This test must check:
          * whether the persona can list all secret stores
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret_store(self):
        """Test get secret store information

        Testing: GET /v1/secret-stores/{secret-store-id}
        This test must check:
          * whether the persona can get information about a specific
            secret store
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_global_secret_store(self):
        """Test getting the global secret store

        Testing: GET /v1/secret-stores/global-default
        This test must check:
          * whether the persona can get information about the global
            default secret store
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_preferred_secret_store(self):
        """Test getting the preferred secret store

        Testing: GET /v1/secret-stores/preferred
        This test must check:
          * whether the persona can get information about their project's
            preferred secret store
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_set_preferred_secret_store(self):
        """Test setting the preferred secret store

        Testing: POST /v1/secret-stores/{secret-store-id}/preferred
        This test must check:
          * whether the persona can set their project's preferred
            secret store
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_unset_preferred_secret_store(self):
        """Test removing the preferred secret store

        Testing: DELETE /v1/secret-stores/{secret-store-id}/preferred
        This test must check:
          * whether the persona can set their project's preferred
            secret store
        """
        raise NotImplementedError


class ProjectMemberTests(base.BarbicanV1RbacBase, BarbicanV1RbacSecretStores):

    @classmethod
    def skip_checks(cls):
        """TODO(redrobot): Run this with multiple backends

        We need to set up the devstack plugin to use multiple backends
        so we can run these tests.
        """
        if not CONF.barbican_tempest.enable_multiple_secret_stores:
            raise cls.skipException("enable_multiple_secret_stores is not "
                                    "configured.  Skipping RBAC tests.")

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.secret_v1.SecretStoresClient()

    def test_list_secret_stores(self):
        resp = self.do_request('list_secret_stores')
        self.assertIn('secret_stores', resp)

    def test_get_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        resp = self.do_request('get_secret_store',
                               secret_store_id=secret_store_id)
        self.assertEqual(secret_store_id,
                         self.ref_to_uuid(resp['secret_store_ref']))

    def test_get_global_secret_store(self):
        resp = self.do_request('get_global_secret_store')
        self.assertTrue(resp['global_default'])

    def test_get_preferred_secret_store(self):
        resp = self.do_request('get_preferred_secret_store')
        self.assertEqual('ACTIVE', resp['status'])

    def test_set_preferred_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        self.do_request('set_preferred_secret_store',
                        expected_status=exceptions.Forbidden,
                        secret_store_id=secret_store_id)

    def test_unset_preferred_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        self.do_request('unset_peferred_secret_store',
                        expected_status=exceptions.Forbidden,
                        secret_store_id=secret_store_id)


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.secret_v1.SecretStoresClient()

    def test_set_preferred_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        self.do_request('set_preferred_secret_store',
                        secret_store_id=secret_store_id)
        resp = self.do_request('get_preferred_secret_store')
        self.assertEqual(secret_store_id,
                         self.ref_to_uuid(resp['secret_store_ref']))

    def test_unset_preferred_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        self.do_request('set_preferred_secret_store',
                        secret_store_id=secret_store_id)
        self.do_request('unset_peferred_secret_store',
                        secret_store_id=secret_store_id)
        resp = self.do_request('get_preferred_secret_store')
        self.assertEqual(secret_store_id,
                         self.ref_to_uuid(resp['secret_store_ref']))


class ProjectReaderTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.SecretStoresClient()



# Copyright (c) 2016 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json
from urllib import parse as urllib

from tempest import config

from barbican_tempest_plugin.services.key_manager.json import base

CONF = config.CONF


class ContainerClient(base.BarbicanTempestClient):

    def list_containers(self, **kwargs):
        uri = "v1/containers"
        if kwargs:
            uri += "?%s" % urllib.urlencode(kwargs)

        response, body = self.get(uri)
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))

    def get_container(self, container_id):
        uri = "v1/containers/%s" % container_id

        response, body = self.get(uri)
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))

    def create_container(self, **kwargs):
        uri = "v1/containers"

        response, body = self.post(uri, json.dumps(kwargs))
        self.expected_success(201, response.status)
        return json.loads(body.decode("utf-8"))

    def delete_container(self, container_id):
        uri = "v1/containers/%s" % container_id

        response, _ = self.delete(uri)
        self.expected_success(204, response.status)
        return

    def add_secret_to_container(self, container_id, secret_id, **kwargs):
        uri = "v1/containers/%s/secrets" % container_id
        kwargs['secret_ref'] = "%s/v1/secrets/%s" % (
            self.auth_provider.base_url({"service": "key-manager"}),
            secret_id
        )

        response, body = self.post(
            uri,
            json.dumps(kwargs)
        )
        self.expected_success(201, response.status)
        return json.loads(body.decode("utf-8"))

    def delete_secret_from_container(self, container_id, secret_id, **kwargs):
        uri = "v1/containers/%s/secrets" % container_id
        kwargs['secret_ref'] = "%s/v1/secrets/%s" % (
            self.auth_provider.base_url({"service": "key-manager"}),
            secret_id
        )

        response, _ = self.delete(
            uri,
            body=json.dumps(kwargs)
        )
        self.expected_success(204, response.status)
        return



# Copyright 2017 Johns Hopkins Applied Physics Lab
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base64
from datetime import datetime
from datetime import timedelta
import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.asymmetric import utils
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography import x509
from cryptography.x509.oid import NameOID

from oslo_log import log as logging
from tempest.api.compute import api_microversion_fixture
from tempest import config

from barbican_tempest_plugin.tests.scenario import manager as mgr

CONF = config.CONF
LOG = logging.getLogger(__name__)
COMPUTE_MICROVERSION = CONF.compute.min_microversion


class BarbicanScenarioTest(mgr.ScenarioTest):

    api_microversion_header_name = 'X-OpenStack-Nova-API-Version'
    credentials = ('primary', 'admin')

    def get_headers(self):
        headers = super(BarbicanScenarioTest, self).get_headers()
        if COMPUTE_MICROVERSION:
            headers[self.api_microversion_header_name] = COMPUTE_MICROVERSION
        return headers

    def setUp(self):
        super(BarbicanScenarioTest, self).setUp()
        self.useFixture(api_microversion_fixture.APIMicroversionFixture(
            self.request_microversion))
        self.img_file = CONF.scenario.img_file
        if not os.path.exists(self.img_file):
            # TODO(kopecmartin): replace LOG.warning for rasing
            # InvalidConfiguration exception after tempest 25.0.0 is
            # released - there will be one release which accepts both
            # behaviors in order to avoid many failures across CIs and etc.
            LOG.warning(
                'Starting Tempest 25.0.0 release, CONF.scenario.img_file need '
                'a full path for the image. CONF.scenario.img_dir was '
                'deprecated and will be removed in the next release. Till '
                'Tempest 25.0.0, old behavior is maintained and keep working '
                'but starting Tempest 26.0.0, you need to specify the full '
                'path in CONF.scenario.img_file config option.')
            self.img_file = os.path.join(CONF.scenario.img_dir, self.img_file)

        self.private_key = rsa.generate_private_key(public_exponent=65537,
                                                    key_size=3072,
                                                    backend=default_backend())
        self.signing_certificate = self._create_self_signed_certificate(
            self.private_key,
            u"Test Certificate"
        )
        self.signing_cert_uuid = self._store_cert(
            self.signing_certificate
        )
        self.bad_signing_certificate = self._create_self_signed_certificate(
            self.private_key,
            u"Bad Certificate"
        )
        self.bad_cert_uuid = self._store_cert(
            self.bad_signing_certificate
        )

    @classmethod
    def skip_checks(cls):
        super(BarbicanScenarioTest, cls).skip_checks()
        if not CONF.service_available.barbican:
            raise cls.skipException('Barbican is not enabled.')

    @classmethod
    def setup_clients(cls):
        super(BarbicanScenarioTest, cls).setup_clients()

        os = getattr(cls, 'os_%s' % cls.credentials[0])
        os_adm = getattr(cls, 'os_%s' % cls.credentials[1])
        cls.consumer_client = os.secret_v1.ConsumerClient(
            service='key-manager'
        )
        cls.container_client = os.secret_v1.ContainerClient(
            service='key-manager'
        )
        cls.order_client = os.secret_v1.OrderClient(service='key-manager')
        cls.secret_client = os.secret_v1.SecretClient(service='key-manager')
        cls.secret_metadata_client = os.secret_v1.SecretMetadataClient(
            service='key-manager'
        )

        if CONF.compute_feature_enabled.attach_encrypted_volume:
            cls.admin_volume_types_client =\
                os_adm.volume_types_v2_client
            cls.admin_encryption_types_client =\
                os_adm.encryption_types_v2_client

    def _get_uuid(self, href):
        return href.split('/')[-1]

    def _create_self_signed_certificate(self, private_key, common_name):
        issuer = x509.Name([
            x509.NameAttribute(NameOID.COUNTRY_NAME, u"US"),
            x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, u"CA"),
            x509.NameAttribute(NameOID.LOCALITY_NAME, u"San Francisco"),
            x509.NameAttribute(NameOID.ORGANIZATION_NAME, u"My Company"),
            x509.NameAttribute(NameOID.COMMON_NAME, common_name),
        ])
        cert_builder = x509.CertificateBuilder(
            issuer_name=issuer,
            subject_name=issuer,
            public_key=private_key.public_key(),
            serial_number=x509.random_serial_number(),
            not_valid_before=datetime.utcnow(),
            not_valid_after=datetime.utcnow() + timedelta(days=10)
        ).add_extension(
            x509.BasicConstraints(
                ca=True,
                path_length=1
            ),
            critical=True
        ).add_extension(
            x509.KeyUsage(
                digital_signature=True,
                content_commitment=True,
                key_encipherment=False,
                data_encipherment=False,
                key_agreement=False,
                key_cert_sign=True,
                crl_sign=False,
                encipher_only=False,
                decipher_only=False
            ),
            critical=False
        )
        cert = cert_builder.sign(private_key,
                                 hashes.SHA256(),
                                 default_backend())
        return cert

    def _store_cert(self, cert):
        pem_encoding = cert.public_bytes(encoding=serialization.Encoding.PEM)
        cert_b64 = base64.b64encode(pem_encoding)
        expire_time = (datetime.utcnow() + timedelta(days=5))
        LOG.debug("Uploading certificate to barbican")
        result = self.secret_client.create_secret(
            expiration=expire_time.isoformat(), algorithm="rsa",
            secret_type="certificate",
            payload_content_type="application/octet-stream",
            payload_content_encoding="base64",
            payload=cert_b64
        )
        LOG.debug("Certificate uploaded to barbican (%s)", result)
        return self._get_uuid(result['secret_ref'])

    def _sign_image(self, image_file):
        LOG.debug("Creating signature for image data")
        hasher = hashes.Hash(hashes.SHA256(), default_backend())
        chunk_bytes = 8192
        with open(image_file, 'rb') as f:
            chunk = f.read(chunk_bytes)
            while len(chunk) > 0:
                hasher.update(chunk)
                chunk = f.read(chunk_bytes)
        digest = hasher.finalize()
        signature = self.private_key.sign(
            digest,
            padding.PSS(
                mgf=padding.MGF1(hashes.SHA256()),
                salt_length=padding.PSS.MAX_LENGTH
            ),
            utils.Prehashed(hashes.SHA256())
        )
        signature_b64 = base64.b64encode(signature)
        return signature_b64

    def sign_and_upload_image(self):
        img_signature = self._sign_image(self.img_file)

        img_properties = {
            'img_signature': img_signature,
            'img_signature_certificate_uuid': self.signing_cert_uuid,
            'img_signature_key_type': 'RSA-PSS',
            'img_signature_hash_method': 'SHA-256',
        }

        LOG.debug("Uploading image with signature metadata properties")
        img_uuid = self._image_create(
            'signed_img',
            CONF.scenario.img_container_format,
            self.img_file,
            disk_format=CONF.scenario.img_disk_format,
            properties=img_properties
        )
        LOG.debug("Uploaded image %s", img_uuid)

        return img_uuid

    def create_encryption_type(self, client=None, type_id=None, provider=None,
                               key_size=None, cipher=None,
                               control_location=None):
        if not client:
            client = self.admin_encryption_types_client
        if not type_id:
            volume_type = self.create_volume_type()
            type_id = volume_type['id']
        LOG.debug("Creating an encryption type for volume type: %s", type_id)
        client.create_encryption_type(
            type_id, provider=provider, key_size=key_size, cipher=cipher,
            control_location=control_location)



# Copyright (c) 2017 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest import exceptions
from tempest.lib.common import api_version_utils
from tempest.lib import decorators

from barbican_tempest_plugin.tests.scenario import barbican_manager

CONF = config.CONF
LOG = logging.getLogger(__name__)


class CertificateValidationTest(barbican_manager.BarbicanScenarioTest):
    min_microversion = '2.63'
    max_microversion = 'latest'

    @classmethod
    def resource_setup(cls):
        super(CertificateValidationTest, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.compute.min_microversion))

    @classmethod
    def skip_checks(cls):
        super(CertificateValidationTest, cls).skip_checks()
        api_version_utils.check_skip_with_microversion(
            cls.min_microversion,
            cls.max_microversion,
            CONF.compute.min_microversion,
            CONF.compute.max_microversion)
        if not CONF.auth.create_isolated_networks:
            # FIXME(redorobt): remove this skip when system-scope admin
            # issue is fixed.
            raise cls.skipException(
                'Certificate Validation tests require isolated networks')

    @decorators.idempotent_id('b41bc663-5662-4b1e-b8f1-27b2876f16a6')
    @utils.services('compute', 'image')
    def test_signed_image_upload_and_boot(self):
        """Test that Nova boots a signed image.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Boot the signed image with a valid trusted image certificate ID
            * Confirm the instance changes state to Active
        """
        img_uuid = self.sign_and_upload_image()

        LOG.debug("Booting server with self-signed image %s and certificate "
                  "ID %s", img_uuid, self.signing_cert_uuid)
        instance = self.create_server(name='signed_img_server',
                                      image_id=img_uuid,
                                      wait_until='ACTIVE',
                                      trusted_image_certificates=[
                                          self.signing_cert_uuid])
        self.servers_client.delete_server(instance['id'])

    @decorators.idempotent_id('6d354881-35a6-4568-94b8-2204bbf67b29')
    @utils.services('compute', 'image')
    def test_signed_image_invalid_cert_boot_failure(self):
        """Test that Nova refuses to boot an unvalidated signed image.

        If the create_server call succeeds instead of throwing an
        exception, it is likely that certificate validation is not
        turned on.  To turn on certificate validation, set
        enable_certificate_validation=True in the nova configuration
        file under the [glance] section.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Attempt to boot the signed image with an invalid trusted
              image certificate ID
            * Confirm an exception is thrown
        """
        img_uuid = self.sign_and_upload_image()

        LOG.debug("Booting server with self-signed image %s and invalid "
                  "certificate ID %s", img_uuid, self.bad_cert_uuid)
        self.assertRaisesRegex(exceptions.BuildErrorException,
                               "Certificate chain building failed",
                               self.create_server,
                               image_id=img_uuid,
                               trusted_image_certificates=[self.bad_cert_uuid])

    @decorators.idempotent_id('aed5254d-1e7a-46b6-8cb0-ef5fd798671a')
    @utils.services('compute', 'image')
    def test_signed_image_upload_and_hard_reboot(self):
        """Test that Nova boots a signed image with certs after a hard reboot.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Boot the signed image with a valid trusted image certificate ID
            * Reboot the signed image
            * Confirm the instance changes state to Active
        """
        img_uuid = self.sign_and_upload_image()

        LOG.debug("Booting server with self-signed image %s and certificate "
                  "ID %s", img_uuid, self.signing_cert_uuid)
        instance = self.create_server(name='server_to_reboot',
                                      image_id=img_uuid,
                                      wait_until='ACTIVE',
                                      trusted_image_certificates=[
                                          self.signing_cert_uuid])

        LOG.debug("Hard rebooting server with self-signed image %s and "
                  "certificate ID %s", img_uuid, self.signing_cert_uuid)
        self.servers_client.reboot_server(instance['id'], type='HARD')
        waiters.wait_for_server_status(self.servers_client, instance['id'],
                                       'ACTIVE')
        self.servers_client.delete_server(instance['id'])

    @decorators.idempotent_id('f9c6de51-b027-476f-a6e3-847bb39cfa02')
    @utils.services('compute', 'image')
    def test_signed_image_upload_and_server_rebuild(self):
        """Test that Nova boots a signed image with certs after a rebuild.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Boot the server with the first signed image
            * Build a second signed image
            * Rebuild the server with the second signed image with a valid
              trusted image certificate ID
            * Confirm the instance changes state to Active
        """
        img_uuid_create = self.sign_and_upload_image()

        LOG.debug("Booting server with self-signed image %s and certificate "
                  "ID %s", img_uuid_create, self.signing_cert_uuid)
        instance = self.create_server(name='server_to_rebuild',
                                      image_id=img_uuid_create,
                                      wait_until='ACTIVE')

        img_uuid_rebuild = self.sign_and_upload_image()
        LOG.debug("Rebuild server with self-signed image %s and certificate "
                  "ID %s", img_uuid_rebuild, self.signing_cert_uuid)
        rebuild_kwargs = {
            'trusted_image_certificates': [self.signing_cert_uuid],
        }
        self.rebuild_server(instance['id'],
                            img_uuid_rebuild,
                            rebuild_kwargs=rebuild_kwargs)
        self.servers_client.delete_server(instance['id'])



# Copyright 2016 SAP SE
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

service_option = cfg.BoolOpt("barbican",
                             default=True,
                             help="Whether or not barbican is expected to be "
                                  "available")

barbican_tempest_group = cfg.OptGroup(
    name='barbican_tempest',
    title='Key Manager (Barbican) service options'
)

BarbicanGroupOpts = [
    cfg.BoolOpt('enable_multiple_secret_stores',
                default=False,
                help="Flag to enable mulitple secret store tests")
]

ephemeral_storage_encryption_group = cfg.OptGroup(
    name="ephemeral_storage_encryption",
    title="Ephemeral storage encryption options")

EphemeralStorageEncryptionGroup = [
    cfg.BoolOpt('enabled',
                default=False,
                help="Does the test environment support ephemeral storage "
                     "encryption?"),
    cfg.StrOpt('cipher',
               default='aes-xts-plain64',
               help="The cipher and mode used to encrypt ephemeral storage. "
                    "AES-XTS is recommended by NIST specifically for disk "
                    "storage, and the name is shorthand for AES encryption "
                    "using the XTS encryption mode. Available ciphers depend "
                    "on kernel support. At the command line, type "
                    "'cryptsetup benchmark' to determine the available "
                    "options (and see benchmark results), or go to "
                    "/proc/crypto."),
    cfg.IntOpt('key_size',
               default=256,
               help="The key size used to encrypt ephemeral storage."),
]

image_signature_verification_group = cfg.OptGroup(
    name="image_signature_verification",
    title="Image Signature Verification Options")

ImageSignatureVerificationGroup = [
    cfg.BoolOpt('enforced',
                default=True,
                help="Does the test environment enforce glance image "
                     "verification?"),
]

barbican_rbac_scope_verification_group = cfg.OptGroup(
    name="barbican_rbac_scope_verification",
    title="Barbican RBAC Verification Options")

BarbicanRBACScopeVerificationGroup = [
    cfg.BoolOpt('enforce_scope',
                default=False,
                help="Does barbican enforce scope and user "
                     "scope-aware policies?"),
]



# Copyright (c) 2016 SAP SE
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from tempest import config
from tempest.lib.services import clients as cli

CONF = config.CONF


class Clients(cli.ServiceClients):
    """Tempest stable service clients and loaded plugins service clients"""

    def __init__(self, credentials, service=None):
        """Emulate the interface of Tempest's clients.Manager"""
        # Identity settings
        if CONF.identity.auth_version == 'v2':
            identity_uri = CONF.identity.uri
        else:
            identity_uri = CONF.identity.uri_v3
        super(Clients, self).__init__(credentials, identity_uri)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import abc

from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


class BarbicanV1RbacTransportKeys:

    @abc.abstractmethod
    def test_list_transport_keys(self):
        """Test listing the transport keys

        Testing: GET /v1/transport_keys
        This test case must check:
          * whether the persona can list the available transport keys
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_create_transport_key(self):
        """Test creating a transport key

        Testing: POST /v1/transport_keys
        This test case must check:
          * whether the persona can create a new transport key entry
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_transport_key(self):
        """Test getting a specific transport key

        Testing: GET /v1/transport_keys/{transport-key-id}
        This test case must check:
          * whether the persona can retrieve a specific transport key
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_transport_key(self):
        """Test deleting a specific transport key

        Testing: DELETE /v1/transport_keys/{transport-key-id}
        This test case must check:
          * whether the persona can delete a specific transport key
        """
        raise NotImplementedError


class ProjectMemberTests(base.BarbicanV1RbacBase, BarbicanV1RbacTransportKeys):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.secret_v1.TransportKeyClient()

    def test_list_transport_keys(self):
        resp = self.do_request('list_transport_keys')
        self.assertIn('transport_keys', resp)

    def test_create_transport_key(self):
        self.do_request('create_transport_key',
                        expected_status=exceptions.Forbidden,
                        plugin_name='simple-crypto',
                        transport_key='???')

    def test_get_transport_key(self):
        # TODO(redorobot):
        # We need to sort out how system admins create keys before we
        # can test this.
        #
        # resp = self.do_request('list_transport_keys')
        # transport_key_id = self.ref_to_uuid(
        #     resp['transport_keys'][0]['transport_key_ref']
        # )
        # resp = self.do_request('get_transport_key',
        #                        transport_key_id=transport_key_id)
        # self.assertEqual(transport_key_id, resp['transport_key_id'])
        pass

    def test_delete_transport_key(self):
        # TODO(redorobot):
        # We need to sort out how system admins create keys before we
        # can test this.
        #
        # resp = self.do_request('list_transport_keys')
        # transport_key_id = self.ref_to_uuid(
        #     resp['transport_keys'][0]['transport_key_ref']
        # )
        # resp = self.do_request('delete_transport_key',
        #                        expected_status=exceptions.Forbidden,
        #                        transport_key_id=transport_key_id)
        pass


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.secret_v1.TransportKeyClient()


class ProjectReaderTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.TransportKeyClient()



# Copyright 2016 SAP SE
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base64
from datetime import datetime
from datetime import timedelta
import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

from tempest.lib import decorators

from barbican_tempest_plugin.tests.api import base


class SecretsTest(base.BaseKeyManagerTest):
    """Secrets API tests."""
    @decorators.idempotent_id('d5fb4ae4-c418-4405-9701-95fc6877aeb9')
    def test_create_delete_empty_secret(self):
        sec = self.create_secret()
        uuid = base._get_uuid(sec['secret_ref'])
        self.delete_secret(uuid)

    @decorators.idempotent_id('9aee2ad3-5b61-4451-8ccc-a727bbe4618a')
    def test_create_delete_symmetric_key(self):
        password = b"password"
        salt = os.urandom(16)
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(), length=32, salt=salt,
            iterations=1000, backend=default_backend()
        )
        key = base64.b64encode(kdf.derive(password))
        expire_time = (datetime.utcnow() + timedelta(days=5))
        sec = self.create_secret(
            expiration=expire_time.isoformat(), algorithm="aes",
            bit_length=256, mode="cbc", payload=key,
            payload_content_type="application/octet-stream",
            payload_content_encoding="base64"
        )
        uuid = base._get_uuid(sec['secret_ref'])
        self.delete_secret(uuid)

    @decorators.idempotent_id('79ec555d-215d-4006-bcf0-ab4c6cb0b9ff')
    def test_list_secrets(self):
        # Create two secrets
        self.create_secret(name='secret_1')
        self.create_secret(name='secret_2')

        # Ask Barbican to list these secrets
        resp = self.secret_client.list_secrets(name='secret_1')
        secrets = resp['secrets']
        self.assertEqual('secret_1', secrets[0]['name'])

        resp = self.secret_client.list_secrets(name='secret_2')
        secrets = resp['secrets']
        self.assertEqual('secret_2', secrets[0]['name'])

    @decorators.idempotent_id('f5608620-f1f7-45a5-ac0a-e1c17d1f2f42')
    def test_get_secret_metadata(self):
        secret = self.create_secret()
        uuid = base._get_uuid(secret['secret_ref'])
        resp = self.secret_client.get_secret_metadata(uuid)
        self.assertEqual(uuid, base._get_uuid(resp['secret_ref']))
        self.delete_secret(uuid)

    @decorators.idempotent_id('c5caa619-1e43-4724-8d94-a61ff7025a07')
    def test_get_and_put_payload(self):
        # Create secret without payload
        secret = self.create_secret()
        uuid = base._get_uuid(secret['secret_ref'])

        # Create AES key payload
        password = b"password"
        salt = os.urandom(16)
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(), length=32, salt=salt,
            iterations=1000, backend=default_backend()
        )
        key = base64.b64encode(kdf.derive(password))

        # Associate the payload with the created secret
        self.secret_client.put_secret_payload(uuid, key)

        # Retrieve the payload
        payload = self.secret_client.get_secret_payload(uuid)
        self.assertEqual(key, base64.b64encode(payload))

        # Clean up
        self.delete_secret(uuid)



# Copyright 2016 SAP SE
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import os

from tempest.test_discover import plugins

from barbican_tempest_plugin import config as project_config


class BarbicanTempestPlugin(plugins.TempestPlugin):
    def load_tests(self):
        base_path = os.path.split(os.path.dirname(
            os.path.abspath(__file__)))[0]
        test_dir = "barbican_tempest_plugin/tests"
        full_test_dir = os.path.join(base_path, test_dir)
        return full_test_dir, base_path

    def register_opts(self, conf):
        conf.register_opt(project_config.service_option,
                          group='service_available')

        conf.register_group(project_config.barbican_tempest_group)
        conf.register_opts(project_config.BarbicanGroupOpts,
                           project_config.barbican_tempest_group)

        # Register ephemeral storage encryption options
        conf.register_group(project_config.ephemeral_storage_encryption_group)
        conf.register_opts(project_config.EphemeralStorageEncryptionGroup,
                           project_config.ephemeral_storage_encryption_group)
        conf.register_opts(project_config.ImageSignatureVerificationGroup,
                           project_config.image_signature_verification_group)
        conf.register_group(
            project_config.barbican_rbac_scope_verification_group)
        conf.register_opts(
            project_config.BarbicanRBACScopeVerificationGroup,
            project_config.barbican_rbac_scope_verification_group
        )

    def get_opt_lists(self):
        return [('service_available', [project_config.service_option])]

    def get_service_clients(self):
        v1_params = {
            'name': 'secret_v1',
            'service_version': 'secret.v1',
            'module_path': 'barbican_tempest_plugin.services.key_manager.json',
            'client_names': [
                'ConsumerClient',
                'ContainerClient',
                'OrderClient',
                'QuotaClient',
                'SecretClient',
                'SecretMetadataClient',
                'SecretStoresClient',
                'TransportKeyClient'
            ],
        }
        return [v1_params]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import abc

from tempest import config
from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


CONF = config.CONF


class BarbicanV1RbacContainers:

    @abc.abstractmethod
    def test_list_containers(self):
        """Test list_containers policy

        Testing: GET /v1/containers
        This test must check:
          * whether the persona can list containers
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_create_container(self):
        """Test create_container policy

        Testing: POST /v1/containers
        Thist test must check:
          * whether the persona can create a new container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_container(self):
        """Test get_container policy

        Testing: GET /v1/containers/{container-id}
        Thist test must check:
          * whether the persona can get a container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_container(self):
        """Test delete_container policy

        Testing: DELETE /v1/containers/{container-id}
        Thist test must check:
          * whether the persona can delete a container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_add_secret_to_container(self):
        """Test add_secret_to_container policy

        Testing: POST /v1/containers/{container-id}/secrets
        Thist test must check:
          * whether the persona can add a secret to a container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_secret_from_container(self):
        """Test delete_secret_from_container policy

        Testing: DELETE /v1/containers/{container-id}/secrets
        Thist test must check:
          * whether the persona can delete a secret from a container
        """
        raise NotImplementedError


class ProjectMemberTests(base.BarbicanV1RbacBase, BarbicanV1RbacContainers):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.secret_v1.ContainerClient()
        cls.secret_client = cls.os_project_member.secret_v1.SecretClient()

    def test_list_containers(self):
        self.do_request('create_container', cleanup='container',
                        name='list_containers', type='generic')

        resp = self.do_request('list_containers')
        containers = resp['containers']

        self.assertGreaterEqual(len(containers), 1)

    def test_create_container(self):
        self.do_request('create_container', cleanup='container',
                        name='create_container', type='generic')

    def test_get_container(self):
        resp = self.do_request('create_container', cleanup='container',
                               name='get_container', type='generic')
        container_id = self.ref_to_uuid(resp['container_ref'])

        resp = self.do_request('get_container', container_id=container_id)

        self.assertEqual(container_id, self.ref_to_uuid(resp['container_ref']))

    def test_delete_container(self):
        resp = self.do_request('create_container', name='delete_container',
                               type='generic')
        container_id = self.ref_to_uuid(resp['container_ref'])

        resp = self.do_request('delete_container', container_id=container_id)

    def test_add_secret_to_container(self):
        resp = self.do_request('create_container', cleanup='container',
                               name='add_secret_to_container_c',
                               type='generic')
        container_id = self.ref_to_uuid(resp['container_ref'])

        resp = self.do_request(
            'create_secret',
            client=self.secret_client,
            cleanup='secret',
            name='add_secret_to_container_s',
            secret_type='passphrase',
            payload='shhh... secret',
            payload_content_type='text/plain'
        )
        secret_id = self.ref_to_uuid(resp['secret_ref'])

        resp = self.do_request('add_secret_to_container',
                               container_id=container_id,
                               secret_id=secret_id)

    def test_delete_secret_from_container(self):
        resp = self.do_request('create_container', cleanup='container',
                               name='delete_secret_from_container_c',
                               type='generic')
        container_id = self.ref_to_uuid(resp['container_ref'])

        resp = self.do_request(
            'create_secret',
            client=self.secret_client,
            cleanup='secret',
            name='delete_secret_from_container_s',
            secret_type='passphrase',
            payload='shhh... secret',
            payload_content_type='text/plain'
        )
        secret_id = self.ref_to_uuid(resp['secret_ref'])

        self.do_request('add_secret_to_container',
                        container_id=container_id,
                        secret_id=secret_id)
        resp = self.do_request('delete_secret_from_container',
                               container_id=container_id,
                               secret_id=secret_id)


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.secret_v1.ContainerClient()
        cls.secret_client = cls.os_project_admin.secret_v1.SecretClient()


class ProjectReaderTests(base.BarbicanV1RbacBase, BarbicanV1RbacContainers):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.ContainerClient()

    def test_list_containers(self):
        self.do_request('list_containers',
                        expected_status=exceptions.Forbidden)

    def test_create_container(self):
        self.do_request('create_container',
                        expected_status=exceptions.Forbidden,
                        name='create_container',
                        type='generic')

    def test_get_container(self):
        resp = self.do_request(
            'create_container',
            client=self.os_project_member.secret_v1.ContainerClient(),
            cleanup='container',
            name='create_container', type='generic'
        )
        container_id = self.ref_to_uuid(resp['container_ref'])

        self.do_request('get_container', expected_status=exceptions.Forbidden,
                        container_id=container_id)

    def test_delete_container(self):
        resp = self.do_request(
            'create_container',
            client=self.os_project_member.secret_v1.ContainerClient(),
            cleanup='container',
            name='delete_container', type='generic'
        )
        container_id = self.ref_to_uuid(resp['container_ref'])

        self.do_request('delete_container',
                        expected_status=exceptions.Forbidden,
                        container_id=container_id)

    def test_add_secret_to_container(self):
        resp = self.do_request(
            'create_container',
            client=self.os_project_member.secret_v1.ContainerClient(),
            cleanup='container',
            name='add_secret_to_container_c', type='generic'
        )
        container_id = self.ref_to_uuid(resp['container_ref'])

        resp = self.do_request(
            'create_secret',
            client=self.os_project_member.secret_v1.SecretClient(),
            cleanup='secret',
            name='add_secret_to_container_s',
            secret_type='passphrase',
            payload='shhh... secret',
            payload_content_type='text/plain'
        )
        secret_id = self.ref_to_uuid(resp['secret_ref'])

        self.do_request('add_secret_to_container',
                        expected_status=exceptions.Forbidden,
                        container_id=container_id,
                        secret_id=secret_id)

    def test_delete_secret_from_container(self):
        resp = self.do_request(
            'create_container',
            client=self.os_project_member.secret_v1.ContainerClient(),
            cleanup='container',
            name='delete_secret_from_container_c', type='generic'
        )
        container_id = self.ref_to_uuid(resp['container_ref'])

        resp = self.do_request(
            'create_secret',
            client=self.os_project_member.secret_v1.SecretClient(),
            cleanup='secret',
            name='delete_secret_from_container_s',
            secret_type='passphrase',
            payload='shhh... secret',
            payload_content_type='text/plain'
        )
        secret_id = self.ref_to_uuid(resp['secret_ref'])

        self.do_request(
            'add_secret_to_container',
            client=self.os_project_member.secret_v1.ContainerClient(),
            container_id=container_id,
            secret_id=secret_id
        )

        self.do_request('delete_secret_from_container',
                        expected_status=exceptions.Forbidden,
                        container_id=container_id,
                        secret_id=secret_id)



# Copyright (c) 2017 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from oslo_log import log as logging
from tempest.common import utils
from tempest import config
from tempest.lib.common import api_version_utils
from tempest.lib import decorators

from barbican_tempest_plugin.tests.scenario import barbican_manager

CONF = config.CONF
LOG = logging.getLogger(__name__)


class VolumeEncryptionTest(barbican_manager.BarbicanScenarioTest):
    min_microversion = '2.1'

    """The test suite for encrypted cinder volumes

    This test is for verifying the functionality of encrypted cinder volumes.
    For both LUKS and cryptsetup encryption types, this test performs
    the following:
        * Creates an image in Glance
        * Boots an instance from the image
        * Creates an encryption type (as admin)
        * Creates a volume of that encryption type (as a regular user)
        * Attaches and detaches the encrypted volume to the instance
    NOTE (dane-fichter): These tests use a key stored in Barbican, unlike
    the original volume encryption scenario in Tempest.
    """

    @classmethod
    def skip_checks(cls):
        super(VolumeEncryptionTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.attach_encrypted_volume:
            raise cls.skipException('Encrypted volume attach is not supported')
        if not CONF.auth.create_isolated_networks:
            # FIXME(redorobt): remove this skip when system-scope admin
            # issue is fixed.
            raise cls.skipException(
                'Volume encryption requires isolated networks')

    @classmethod
    def resource_setup(cls):
        super(VolumeEncryptionTest, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.compute.min_microversion))

    def create_encrypted_volume(self, encryption_provider, volume_type):
        volume_type = self.create_volume_type(name=volume_type)
        self.create_encryption_type(type_id=volume_type['id'],
                                    provider=encryption_provider,
                                    key_size=256,
                                    cipher='aes-xts-plain64',
                                    control_location='front-end')
        return self.create_volume(volume_type=volume_type['name'])

    def attach_detach_volume(self, server, volume, keypair):
        # Attach volume
        self.nova_volume_attach(server, volume)

        # Write a timestamp to volume
        server_ip = self.get_server_ip(server)
        timestamp = self.create_timestamp(
            server_ip,
            dev_name=CONF.compute.volume_device_name,
            private_key=keypair['private_key']
        )
        timestamp2 = self.get_timestamp(
            server_ip,
            dev_name=CONF.compute.volume_device_name,
            private_key=keypair['private_key']
        )
        self.assertEqual(timestamp, timestamp2)

    @decorators.idempotent_id('89165fb4-5534-4b9d-8429-97ccffb8f86f')
    @utils.services('compute', 'volume', 'image')
    def test_encrypted_cinder_volumes_luks(self):
        img_uuid = self.sign_and_upload_image()
        LOG.info("Creating keypair and security group")
        keypair = self.create_keypair()
        security_group = self._create_security_group()
        server = self.create_server(
            name='signed_img_server',
            image_id=img_uuid,
            key_name=keypair['name'],
            security_groups=[{'name': security_group['name']}],
            wait_until='ACTIVE'
        )

        # check that instance is accessible before messing with its disks
        self.check_tenant_network_connectivity(
            server, CONF.validation.image_ssh_user, keypair['private_key'])

        volume = self.create_encrypted_volume('luks',
                                              volume_type='luks')
        self.attach_detach_volume(server, volume, keypair)

    @testtools.skipIf(CONF.volume.storage_protocol == 'ceph',
                      'PLAIN encryptor provider is not supported on rbd')
    @decorators.idempotent_id('cbc752ed-b716-4727-910f-956ccf965723')
    @utils.services('compute', 'volume', 'image')
    def test_encrypted_cinder_volumes_cryptsetup(self):
        img_uuid = self.sign_and_upload_image()
        LOG.info("Creating keypair and security group")
        keypair = self.create_keypair()
        security_group = self._create_security_group()
        server = self.create_server(
            name='signed_img_server',
            image_id=img_uuid,
            key_name=keypair['name'],
            security_groups=[{'name': security_group['name']}],
            wait_until='ACTIVE'
        )

        # check that instance is accessible before messing with its disks
        self.check_tenant_network_connectivity(
            server, CONF.validation.image_ssh_user, keypair['private_key'])

        volume = self.create_encrypted_volume('plain',
                                              volume_type='cryptsetup')
        self.attach_detach_volume(server, volume, keypair)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import abc

from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


class BarbicanV1RbacQuota:

    @abc.abstractmethod
    def test_get_effective_project_quota(self):
        """Test getting the effective quota information

        Testing: GET /v1/quotas
        This test must check:
          * whether the persona can retrieve the effective quota for
            their project.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_list_project_quotas(self):
        """Test listing all configured project quotas

        Testing: GET /v1/project-quotas
        This test must check:
          * whether the persona can retrieve all modified quotas for
            the entire system.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_custom_quota_for_project(self):
        """Test getting a custom quota for a specific project

        Testing: GET /v1/project-quotas/{project-id}
        This test must check:
          * whether the persona can retrieve the custom quota for a
            specific project.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_set_new_quota_for_project(self):
        """Test setting a custom quota for a specific project

        Testing: PUT /v1/project-quotas/{project-id}
        This test must check:
          * whether the persona can create custom quotas for a
            specific project.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_remove_custom_quota_for_project(self):
        """Test removing a custom quota for a specific project

        Testing: DELETE /v1/project-quotas/{project-id}
        This test must check:
          * whether the persona can delete custom quotas for a
            specific project.
        """
        raise NotImplementedError


class ProjectMemberTests(base.BarbicanV1RbacBase, BarbicanV1RbacQuota):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.secret_v1.QuotaClient()

    def test_get_effective_project_quota(self):
        resp = self.do_request('get_default_project_quota')
        self.assertIn('quotas', resp)

    def test_list_project_quotas(self):
        self.do_request('list_quotas', expected_status=exceptions.Forbidden)

    def test_get_custom_quota_for_project(self):
        project_id = self.client.tenant_id
        self.do_request('get_project_quota',
                        expected_status=exceptions.Forbidden,
                        project_id=project_id)

    def test_set_new_quota_for_project(self):
        project_id = self.client.tenant_id
        self.do_request('create_project_quota',
                        expected_status=exceptions.Forbidden,
                        project_id=project_id)

    def test_remove_custom_quota_for_project(self):
        project_id = self.client.tenant_id
        self.do_request('delete_project_quota',
                        expected_status=exceptions.Forbidden,
                        project_id=project_id)


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.secret_v1.QuotaClient()


class ProjectReaderTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.QuotaClient()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import abc
import time

from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


class BarbicanV1RbacOrders:

    @abc.abstractmethod
    def test_list_orders(self):
        """Test list_orders policy

        Testing GET /v1/orders
        This test must check:
          * whether persona can list orders
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_create_order(self):
        """Test create_order policy

        Testing POST /v1/orders
        This test must check:
          * whether persona can create orders
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_order(self):
        """Test get_order policy

        Testing GET /v1/orders/{order-id}
        This test must check:
          * whether persona can get order information
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_order(self):
        """Test delete_order policy

        Testing DELETE /v1/orders/{order-id}
        This test must check:
          * whether persona can delete orders
        """
        raise NotImplementedError


class ProjectMemberTests(base.BarbicanV1RbacBase, BarbicanV1RbacOrders):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.secret_v1.OrderClient()

    def test_list_orders(self):
        self.do_request('create_order', cleanup='order',
                        name='list_orders', type='key',
                        meta={
                            'name': 'list_orders_s',
                            'algorithm': 'aes',
                            'bit_length': 256,
                            'mode': 'cbc',
                        })
        resp = self.do_request('list_orders')
        self.assertGreaterEqual(len(resp['orders']), 1)

    def test_create_order(self):
        self.do_request('create_order', cleanup='order',
                        name='create_order', type='key',
                        meta={
                            'name': 'create_orders_s',
                            'algorithm': 'aes',
                            'bit_length': 256,
                            'mode': 'cbc',
                        })

    def test_get_order(self):
        resp = self.do_request('create_order', cleanup='order',
                               name='get_order', type='key',
                               meta={
                                   'name': 'get_order_s',
                                   'algorithm': 'aes',
                                   'bit_length': 256,
                                   'mode': 'cbc',
                               })
        order_id = self.ref_to_uuid(resp['order_ref'])
        resp = self.do_request('get_order', order_id=order_id)
        self.assertEqual(order_id, self.ref_to_uuid(resp['order_ref']))

    def test_delete_order(self):
        resp = self.do_request('create_order',
                               name='delete_order', type='key',
                               meta={
                                   'name': 'delete_order_s',
                                   'algorithm': 'aes',
                                   'bit_length': 256,
                                   'mode': 'cbc',
                               })
        order_id = self.ref_to_uuid(resp['order_ref'])
        while True:
            time.sleep(1)
            resp = self.do_request('get_order', order_id=order_id)
            if 'ACTIVE' == resp['status']:
                self.add_cleanup('secret', resp)
                break

        self.do_request('delete_order', order_id=order_id)


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.secret_v1.OrderClient()


class ProjectReaderTests(base.BarbicanV1RbacBase, BarbicanV1RbacOrders):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.OrderClient()

    def test_list_orders(self):
        self.do_request('list_orders', expected_status=exceptions.Forbidden)

    def test_create_order(self):
        self.do_request('create_order',
                        expected_status=exceptions.Forbidden,
                        cleanup='order',
                        name='create_order', type='key',
                        meta={
                            'name': 'create_orders_s',
                            'algorithm': 'aes',
                            'bit_length': 256,
                            'mode': 'cbc',
                        })

    def test_get_order(self):
        resp = self.do_request(
            'create_order',
            client=self.os_project_member.secret_v1.OrderClient(),
            cleanup='order',
            name='get_order', type='key',
            meta={
                'name': 'get_order_s',
                'algorithm': 'aes',
                'bit_length': 256,
                'mode': 'cbc',
            }
        )
        order_id = self.ref_to_uuid(resp['order_ref'])
        self.do_request('get_order', expected_status=exceptions.Forbidden,
                        order_id=order_id)

    def test_delete_order(self):
        resp = self.do_request(
            'create_order',
            client=self.os_project_member.secret_v1.OrderClient(),
            cleanup='order',
            name='delete_order', type='key',
            meta={
                'name': 'delete_order_s',
                'algorithm': 'aes',
                'bit_length': 256,
                'mode': 'cbc',
            })
        order_id = self.ref_to_uuid(resp['order_ref'])
        self.do_request('delete_order', expected_status=exceptions.Forbidden,
                        order_id=order_id)



# Copyright 2016 SAP SE
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import json

from tempest import config
from tempest.lib.common.utils import data_utils

from barbican_tempest_plugin.services.key_manager.json import base


CONF = config.CONF


class SecretClient(base.BarbicanTempestClient):

    def create_secret(self, **kwargs):
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name("tempest-sec")

        if 'payload' in kwargs and type(kwargs['payload']) is bytes:
            kwargs['payload'] = kwargs['payload'].decode('utf-8')

        post_body = kwargs
        body = json.dumps(post_body)
        resp, body = self.post("v1/secrets", body)
        self.expected_success(201, resp.status)
        return self._parse_resp(body)

    def delete_secret(self, secret_id):
        resp, body = self.delete("v1/secrets/%s" % secret_id)
        self.expected_success(204, resp.status)
        return body

    def list_secrets(self, **kwargs):
        uri = "v1/secrets"
        if kwargs is not None:
            uri = '{base}?'.format(base=uri)

            for key in kwargs.keys():
                uri = '{base}&{name}={value}'.format(
                    base=uri,
                    name=key,
                    value=kwargs[key]
                )
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        return self._parse_resp(body)

    def get_secret_metadata(self, secret_id):
        resp, body = self.get("v1/secrets/%s" % secret_id)
        self.expected_success(200, resp.status)
        return self._parse_resp(body)

    def get_secret_payload(self, secret_id):
        content_headers = {
            "Accept": "application/octet-stream"
        }
        resp, body = self.get("v1/secrets/%s/payload" % secret_id,
                              headers=content_headers)
        self.expected_success(200, resp.status)
        return self._parse_resp(body)

    def put_secret_payload(self, secret_id, payload):
        content_headers = {
            "Content-Type": "application/octet-stream",
            "Content-Encoding": "base64"
        }
        resp, body = self.put("v1/secrets/%s" % secret_id,
                              payload,
                              headers=content_headers)
        self.expected_success(204, resp.status)
        return body



# Copyright (c) 2017 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import testtools

from oslo_log import log as logging
from tempest.api.compute import base as compute_base
from tempest.common import utils
from tempest import config
from tempest import exceptions
from tempest.lib.common import api_version_utils
from tempest.lib import decorators

from barbican_tempest_plugin.tests.scenario import barbican_manager

CONF = config.CONF
LOG = logging.getLogger(__name__)


class ImageSigningTest(barbican_manager.BarbicanScenarioTest):
    min_microversion = '2.1'

    @classmethod
    def resource_setup(cls):
        super(ImageSigningTest, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.compute.min_microversion))

    @decorators.idempotent_id('4343df3c-5553-40ea-8705-0cce73b297a9')
    @utils.services('compute', 'image')
    def test_signed_image_upload_and_boot(self):
        """Test that Nova boots a signed image.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Boot the signed image
            * Confirm the instance changes state to Active
        """
        img_uuid = self.sign_and_upload_image()

        LOG.debug("Booting server with signed image %s", img_uuid)
        instance = self.create_server(name='signed_img_server',
                                      image_id=img_uuid,
                                      wait_until='ACTIVE')
        self.servers_client.delete_server(instance['id'])

    @decorators.idempotent_id('74f022d6-a6ef-4458-96b7-541deadacf99')
    @utils.services('compute', 'image')
    @testtools.skipUnless(CONF.image_signature_verification.enforced,
                          "Image signature verification is not enforced")
    def test_signed_image_upload_boot_failure(self):
        """Test that Nova refuses to boot an incorrectly signed image.

        If the create_server call succeeds instead of throwing an
        exception, it is likely that signature verification is not
        turned on.  To turn on signature verification, set
        verify_glance_signatures=True in the nova configuration
        file under the [glance] section.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Modify the signature to be incorrect
            * Attempt to boot the incorrectly signed image
            * Confirm an exception is thrown
        """

        img_uuid = self.sign_and_upload_image()

        LOG.debug("Modifying image signature to be incorrect")
        patch = [dict(replace='/img_signature', value='fake_signature')]
        self.image_client.update_image(image_id=img_uuid, patch=patch)

        self.assertRaisesRegex(exceptions.BuildErrorException,
                               "Signature verification for the image failed",
                               self.create_server,
                               image_id=img_uuid)


class ImageSigningSnapshotTest(barbican_manager.BarbicanScenarioTest,
                               compute_base.BaseV2ComputeTest):

    @classmethod
    def setup_clients(cls):
        super(ImageSigningSnapshotTest, cls).setup_clients()
        cls.client = cls.servers_client

    @decorators.idempotent_id('f0603dfd-8b2c-44e2-8b0f-d65c87aab257')
    @utils.services('compute', 'image')
    def test_signed_image_upload_boot_snapshot(self):
        """Test that Glance can snapshot an instance using a signed image.

        Verify that a snapshot can be taken of an instance booted from a signed
        image and that the resulting snapshot image has had all image signature
        properties dropped from the original image.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Boot the signed image
            * Confirm the instance changes state to Active
            * Snapshot the running instance
            * Uploading the snapshot and confirm the state moves to ACTIVE
        """
        img_uuid = self.sign_and_upload_image()
        instance = self.create_server(name='signed_img_server_to_snapshot',
                                      image_id=img_uuid,
                                      wait_until='ACTIVE')

        # Snapshot the instance, wait until the snapshot is active
        image = self.create_image_from_server(instance['id'],
                                              wait_until='ACTIVE')

        # Ensure all img_signature image props have been dropped
        signature_props = ['img_signature_hash_method',
                           'img_signature',
                           'img_signature_key_type',
                           'img_signature_certificate_uuid']
        img_meta = self.compute_images_client.list_image_metadata(image['id'])
        self.assertFalse(any(x in img_meta for x in signature_props))

        self.servers_client.delete_server(instance['id'])



# Copyright 2012 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log

from tempest.common import compute
from tempest.common import image as common_image
from tempest.common.utils.linux import remote_client
from tempest.common import waiters
from tempest import config
from tempest import exceptions
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exc
from tempest.scenario import manager

CONF = config.CONF

LOG = log.getLogger(__name__)


# we inherit from NetworkScenarioTest since some test cases need access to
# check_*_connectivity methods to validate instances are up and accessible
class ScenarioTest(manager.NetworkScenarioTest):
    """Base class for scenario tests. Uses tempest own clients. """

    credentials = ['primary']

    @classmethod
    def setup_clients(cls):
        super(ScenarioTest, cls).setup_clients()
        # Clients (in alphabetical order)
        cls.flavors_client = cls.os_primary.flavors_client
        cls.compute_floating_ips_client = (
            cls.os_primary.compute_floating_ips_client)
        if CONF.service_available.glance:
            # Check if glance v1 is available to determine which client to use.
            if CONF.image_feature_enabled.api_v1:
                cls.image_client = cls.os_primary.image_client
            elif CONF.image_feature_enabled.api_v2:
                cls.image_client = cls.os_primary.image_client_v2
            else:
                raise lib_exc.InvalidConfiguration(
                    'Either api_v1 or api_v2 must be True in '
                    '[image-feature-enabled].')
        # Compute image client
        cls.compute_images_client = cls.os_primary.compute_images_client
        cls.keypairs_client = cls.os_primary.keypairs_client
        # Nova security groups client
        cls.compute_security_groups_client = (
            cls.os_primary.compute_security_groups_client)
        cls.compute_security_group_rules_client = (
            cls.os_primary.compute_security_group_rules_client)
        cls.servers_client = cls.os_primary.servers_client
        # Neutron network client
        cls.networks_client = cls.os_primary.networks_client
        cls.ports_client = cls.os_primary.ports_client
        cls.routers_client = cls.os_primary.routers_client
        cls.subnets_client = cls.os_primary.subnets_client
        cls.floating_ips_client = cls.os_primary.floating_ips_client
        cls.security_groups_client = cls.os_primary.security_groups_client
        cls.security_group_rules_client = (
            cls.os_primary.security_group_rules_client)

        cls.volumes_client = cls.os_primary.volumes_client_latest
        cls.snapshots_client = cls.os_primary.snapshots_client_latest

    # ## Test functions library
    #
    # The create_[resource] functions only return body and discard the
    # resp part which is not used in scenario tests

    def _create_port(self, network_id, client=None, namestart='port-quotatest',
                     **kwargs):
        if not client:
            client = self.ports_client
        name = data_utils.rand_name(namestart)
        result = client.create_port(
            name=name,
            network_id=network_id,
            **kwargs)
        self.assertIsNotNone(result, 'Unable to allocate port')
        port = result['port']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        client.delete_port, port['id'])
        return port

    def create_keypair(self, client=None):
        if not client:
            client = self.keypairs_client
        name = data_utils.rand_name(self.__class__.__name__)
        # We don't need to create a keypair by pubkey in scenario
        body = client.create_keypair(name=name)
        self.addCleanup(client.delete_keypair, name)
        return body['keypair']

    def create_server(self, name=None, image_id=None, flavor=None,
                      validatable=False, wait_until='ACTIVE',
                      clients=None, **kwargs):
        """Wrapper utility that returns a test server.

        This wrapper utility calls the common create test server and
        returns a test server. The purpose of this wrapper is to minimize
        the impact on the code of the tests already using this
        function.
        """

        # NOTE(jlanoux): As a first step, ssh checks in the scenario
        # tests need to be run regardless of the run_validation and
        # validatable parameters and thus until the ssh validation job
        # becomes voting in CI. The test resources management and IP
        # association are taken care of in the scenario tests.
        # Therefore, the validatable parameter is set to false in all
        # those tests. In this way create_server just return a standard
        # server and the scenario tests always perform ssh checks.

        # Needed for the cross_tenant_traffic test:
        if clients is None:
            clients = self.os_primary

        if name is None:
            name = data_utils.rand_name(self.__class__.__name__ + "-server")

        vnic_type = CONF.network.port_vnic_type

        # If vnic_type is configured create port for
        # every network
        if vnic_type:
            ports = []

            create_port_body = {'binding:vnic_type': vnic_type,
                                'namestart': 'port-smoke'}
            if kwargs:
                # Convert security group names to security group ids
                # to pass to create_port
                if 'security_groups' in kwargs:
                    security_groups = \
                        clients.security_groups_client.list_security_groups(
                        ).get('security_groups')
                    sec_dict = dict([(s['name'], s['id'])
                                    for s in security_groups])

                    sec_groups_names = [s['name'] for s in kwargs.pop(
                        'security_groups')]
                    security_groups_ids = [sec_dict[s]
                                           for s in sec_groups_names]

                    if security_groups_ids:
                        create_port_body[
                            'security_groups'] = security_groups_ids
                networks = kwargs.pop('networks', [])
            else:
                networks = []

            # If there are no networks passed to us we look up
            # for the project's private networks and create a port.
            # The same behaviour as we would expect when passing
            # the call to the clients with no networks
            if not networks:
                networks = clients.networks_client.list_networks(
                    **{'router:external': False, 'fields': 'id'})['networks']

            # It's net['uuid'] if networks come from kwargs
            # and net['id'] if they come from
            # clients.networks_client.list_networks
            for net in networks:
                net_id = net.get('uuid', net.get('id'))
                if 'port' not in net:
                    port = self._create_port(network_id=net_id,
                                             client=clients.ports_client,
                                             **create_port_body)
                    ports.append({'port': port['id']})
                else:
                    ports.append({'port': net['port']})
            if ports:
                kwargs['networks'] = ports
            self.ports = ports

        tenant_network = self.get_tenant_network()

        body, servers = compute.create_test_server(
            clients,
            tenant_network=tenant_network,
            wait_until=wait_until,
            name=name, flavor=flavor,
            image_id=image_id, **kwargs)

        self.addCleanup(waiters.wait_for_server_termination,
                        clients.servers_client, body['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        clients.servers_client.delete_server, body['id'])
        server = clients.servers_client.show_server(body['id'])['server']
        return server

    def create_volume(self, size=None, name=None, snapshot_id=None,
                      imageRef=None, volume_type=None):
        if size is None:
            size = CONF.volume.volume_size
        if imageRef:
            image = self.compute_images_client.show_image(imageRef)['image']
            min_disk = image.get('minDisk')
            size = max(size, min_disk)
        if name is None:
            name = data_utils.rand_name(self.__class__.__name__ + "-volume")
        kwargs = {'display_name': name,
                  'snapshot_id': snapshot_id,
                  'imageRef': imageRef,
                  'volume_type': volume_type,
                  'size': size}
        if CONF.compute.compute_volume_common_az:
            kwargs.setdefault('availability_zone',
                              CONF.compute.compute_volume_common_az)
        volume = self.volumes_client.create_volume(**kwargs)['volume']

        self.addCleanup(self.volumes_client.wait_for_resource_deletion,
                        volume['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.volumes_client.delete_volume, volume['id'])

        # NOTE(e0ne): Cinder API v2 uses name instead of display_name
        if 'display_name' in volume:
            self.assertEqual(name, volume['display_name'])
        else:
            self.assertEqual(name, volume['name'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        # The volume retrieved on creation has a non-up-to-date status.
        # Retrieval after it becomes active ensures correct details.
        volume = self.volumes_client.show_volume(volume['id'])['volume']
        return volume

    def create_volume_type(self, client=None, name=None, backend_name=None):
        if not client:
            client = self.admin_volume_types_client
        if not name:
            class_name = self.__class__.__name__
            name = data_utils.rand_name(class_name + '-volume-type')
        randomized_name = data_utils.rand_name('scenario-type-' + name)

        LOG.debug("Creating a volume type: %s on backend %s",
                  randomized_name, backend_name)
        extra_specs = {}
        if backend_name:
            extra_specs = {"volume_backend_name": backend_name}

        body = client.create_volume_type(name=randomized_name,
                                         extra_specs=extra_specs)
        volume_type = body['volume_type']
        self.assertIn('id', volume_type)
        self.addCleanup(client.delete_volume_type, volume_type['id'])
        return volume_type

    def _image_create(self, name, fmt, path,
                      disk_format=None, properties=None):
        if properties is None:
            properties = {}
        name = data_utils.rand_name('%s-' % name)
        params = {
            'name': name,
            'container_format': fmt,
            'disk_format': disk_format or fmt,
        }
        if CONF.image_feature_enabled.api_v1:
            params['is_public'] = 'False'
            params['properties'] = properties
            params = {'headers': common_image.image_meta_to_headers(**params)}
        else:
            params['visibility'] = 'private'
            # Additional properties are flattened out in the v2 API.
            params.update(properties)
        body = self.image_client.create_image(**params)
        image = body['image'] if 'image' in body else body
        self.addCleanup(self.image_client.delete_image, image['id'])
        self.assertEqual("queued", image['status'])
        with open(path, 'rb') as image_file:
            if CONF.image_feature_enabled.api_v1:
                self.image_client.update_image(image['id'], data=image_file)
            else:
                self.image_client.store_image_file(image['id'], image_file)

        if CONF.image_feature_enabled.import_image:
            available_stores = []
            try:
                available_stores = self.image_client.info_stores()['stores']
            except exceptions.NotFound:
                pass
            available_import_methods = self.image_client.info_import()[
                'import-methods']['value']
            if ('copy-image' in available_import_methods and
                    len(available_stores) > 1):
                self.image_client.image_import(image['id'],
                                               method='copy-image',
                                               all_stores=True,
                                               all_stores_must_succeed=False)
                failed_stores = waiters.wait_for_image_copied_to_stores(
                    self.image_client, image['id'])
                self.assertEqual(0, len(failed_stores),
                                 "Failed to copy the following stores: %s" %
                                 str(failed_stores))

        return image['id']

    def rebuild_server(self, server_id, image=None,
                       preserve_ephemeral=False, wait=True,
                       rebuild_kwargs=None):
        if image is None:
            image = CONF.compute.image_ref

        rebuild_kwargs = rebuild_kwargs or {}

        LOG.debug("Rebuilding server (id: %s, image: %s, preserve eph: %s)",
                  server_id, image, preserve_ephemeral)
        self.servers_client.rebuild_server(
            server_id=server_id, image_ref=image,
            preserve_ephemeral=preserve_ephemeral,
            **rebuild_kwargs)
        if wait:
            waiters.wait_for_server_status(self.servers_client,
                                           server_id, 'ACTIVE')

    def create_floating_ip(self, thing, pool_name=None):
        """Create a floating IP and associates to a server on Nova"""

        if not pool_name:
            pool_name = CONF.network.floating_network_name
        floating_ip = (self.compute_floating_ips_client.
                       create_floating_ip(pool=pool_name)['floating_ip'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.compute_floating_ips_client.delete_floating_ip,
                        floating_ip['id'])
        self.compute_floating_ips_client.associate_floating_ip_to_server(
            floating_ip['ip'], thing['id'])
        return floating_ip

    def nova_volume_attach(self, server, volume_to_attach):
        volume = self.servers_client.attach_volume(
            server['id'], volumeId=volume_to_attach['id'], device='/dev/%s'
            % CONF.compute.volume_device_name)['volumeAttachment']
        self.assertEqual(volume_to_attach['id'], volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'in-use')

        self.addCleanup(self.nova_volume_detach, server, volume)
        # Return the updated volume after the attachment
        return self.volumes_client.show_volume(volume['id'])['volume']

    def nova_volume_detach(self, server, volume):
        self.servers_client.detach_volume(server['id'], volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

        volume = self.volumes_client.show_volume(volume['id'])['volume']
        self.assertEqual('available', volume['status'])

    def create_timestamp(self, ip_address, dev_name=None, mount_path='/mnt',
                         private_key=None):
        ssh_client = self.get_remote_client(ip_address,
                                            private_key=private_key)
        if dev_name is not None:
            ssh_client.make_fs(dev_name)
            ssh_client.exec_command('sudo mount /dev/%s %s' % (dev_name,
                                                               mount_path))
        cmd_timestamp = 'sudo sh -c "date > %s/timestamp; sync"' % mount_path
        ssh_client.exec_command(cmd_timestamp)
        timestamp = ssh_client.exec_command('sudo cat %s/timestamp'
                                            % mount_path)
        if dev_name is not None:
            ssh_client.exec_command('sudo umount %s' % mount_path)
        return timestamp

    def get_timestamp(self, ip_address, dev_name=None, mount_path='/mnt',
                      private_key=None):
        ssh_client = self.get_remote_client(ip_address,
                                            private_key=private_key)
        if dev_name is not None:
            ssh_client.mount(dev_name, mount_path)
        timestamp = ssh_client.exec_command('sudo cat %s/timestamp'
                                            % mount_path)
        if dev_name is not None:
            ssh_client.exec_command('sudo umount %s' % mount_path)
        return timestamp

    def get_server_ip(self, server):
        """Get the server fixed or floating IP.

        Based on the configuration we're in, return a correct ip
        address for validating that a guest is up.
        """
        if CONF.validation.connect_method == 'floating':
            # The tests calling this method don't have a floating IP
            # and can't make use of the validation resources. So the
            # method is creating the floating IP there.
            return self.create_floating_ip(server)['ip']
        elif CONF.validation.connect_method == 'fixed':
            # Determine the network name to look for based on config or creds
            # provider network resources.
            if CONF.validation.network_for_ssh:
                addresses = server['addresses'][
                    CONF.validation.network_for_ssh]
            else:
                creds_provider = self._get_credentials_provider()
                net_creds = creds_provider.get_primary_creds()
                network = getattr(net_creds, 'network', None)
                addresses = (server['addresses'][network['name']]
                             if network else [])
            for address in addresses:
                ip_version_for_ssh = CONF.validation.ip_version_for_ssh
                if (address['version'] == ip_version_for_ssh and
                        address['OS-EXT-IPS:type'] == 'fixed'):
                    return address['addr']
            raise exceptions.ServerUnreachable(server_id=server['id'])
        else:
            raise lib_exc.InvalidConfiguration()

    def get_remote_client(self, ip_address, username=None, private_key=None):
        """Get a SSH client to a remote server

        @param ip_address the server floating or fixed IP address to use
                          for ssh validation
        @param username name of the Linux account on the remote server
        @param private_key the SSH private key to use
        @return a RemoteClient object
        """

        if username is None:
            username = CONF.validation.image_ssh_user
        # Set this with 'keypair' or others to log in with keypair or
        # username/password.
        if CONF.validation.auth_method == 'keypair':
            password = None
            if private_key is None:
                private_key = self.keypair['private_key']
        else:
            password = CONF.validation.image_ssh_password
            private_key = None
        linux_client = remote_client.RemoteClient(ip_address, username,
                                                  pkey=private_key,
                                                  password=password)
        try:
            linux_client.validate_authentication()
        except Exception as e:
            message = ('Initializing SSH connection to %(ip)s failed. '
                       'Error: %(error)s' % {'ip': ip_address,
                                             'error': e})
            caller = test_utils.find_test_caller()
            if caller:
                message = '(%s) %s' % (caller, message)
            LOG.exception(message)
            self._log_console_output()
            raise

        return linux_client

    def _default_security_group(self, client=None, tenant_id=None):
        """Get default secgroup for given tenant_id.

        :returns: default secgroup for given tenant
        """
        if client is None:
            client = self.security_groups_client
        if not tenant_id:
            tenant_id = client.tenant_id
        sgs = [
            sg for sg in list(client.list_security_groups().values())[0]
            if sg['tenant_id'] == tenant_id and sg['name'] == 'default'
        ]
        msg = "No default security group for tenant %s." % (tenant_id)
        self.assertGreater(len(sgs), 0, msg)
        return sgs[0]

    def _create_security_group(self):
        # Create security group
        sg_name = data_utils.rand_name(self.__class__.__name__)
        sg_desc = sg_name + " description"
        secgroup = self.compute_security_groups_client.create_security_group(
            name=sg_name, description=sg_desc)['security_group']
        self.assertEqual(secgroup['name'], sg_name)
        self.assertEqual(secgroup['description'], sg_desc)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.compute_security_groups_client.delete_security_group,
            secgroup['id'])

        # Add rules to the security group
        self._create_loginable_secgroup_rule(secgroup['id'])

        return secgroup

    def _create_loginable_secgroup_rule(self, secgroup_id=None):
        _client = self.compute_security_groups_client
        _client_rules = self.compute_security_group_rules_client
        if secgroup_id is None:
            sgs = _client.list_security_groups()['security_groups']
            for sg in sgs:
                if sg['name'] == 'default':
                    secgroup_id = sg['id']

        # These rules are intended to permit inbound ssh and icmp
        # traffic from all sources, so no group_id is provided.
        # Setting a group_id would only permit traffic from ports
        # belonging to the same security group.
        rulesets = [
            {
                # ssh
                'ip_protocol': 'tcp',
                'from_port': 22,
                'to_port': 22,
                'cidr': '0.0.0.0/0',
            },
            {
                # ping
                'ip_protocol': 'icmp',
                'from_port': -1,
                'to_port': -1,
                'cidr': '0.0.0.0/0',
            }
        ]
        rules = list()
        for ruleset in rulesets:
            sg_rule = _client_rules.create_security_group_rule(
                parent_group_id=secgroup_id, **ruleset)['security_group_rule']
            rules.append(sg_rule)
        return rules

    def _create_security_group_rule(self, secgroup=None,
                                    sec_group_rules_client=None,
                                    tenant_id=None,
                                    security_groups_client=None, **kwargs):
        """Create a rule from a dictionary of rule parameters.

        Create a rule in a secgroup. if secgroup not defined will search for
        default secgroup in tenant_id.

        :param secgroup: the security group.
        :param tenant_id: if secgroup not passed -- the tenant in which to
            search for default secgroup
        :param kwargs: a dictionary containing rule parameters:
            for example, to allow incoming ssh:
            rule = {
                    direction: 'ingress'
                    protocol:'tcp',
                    port_range_min: 22,
                    port_range_max: 22
                    }
        """
        if sec_group_rules_client is None:
            sec_group_rules_client = self.security_group_rules_client
        if security_groups_client is None:
            security_groups_client = self.security_groups_client
        if not tenant_id:
            tenant_id = security_groups_client.tenant_id
        if secgroup is None:
            secgroup = self._default_security_group(
                client=security_groups_client, tenant_id=tenant_id)

        ruleset = dict(security_group_id=secgroup['id'],
                       tenant_id=secgroup['tenant_id'])
        ruleset.update(kwargs)

        sg_rule = sec_group_rules_client.create_security_group_rule(**ruleset)
        sg_rule = sg_rule['security_group_rule']

        self.assertEqual(secgroup['tenant_id'], sg_rule['tenant_id'])
        self.assertEqual(secgroup['id'], sg_rule['security_group_id'])

        return sg_rule



# Copyright (c) 2016 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from barbican_tempest_plugin.tests.api import base


class ContainersTest(base.BaseKeyManagerTest):
    """Containers API tests."""

    @decorators.idempotent_id('2e13d4bb-54de-463a-a358-0fb9a221d8f3')
    def test_create_list_delete_empty_container(self):
        # Create a container to test against.
        body = self.create_container(type="generic", name="empty-container")
        container_id = base._get_uuid(body.get('container_ref'))

        # Verify that the container can be found via specific listing.
        body = self.container_client.get_container(container_id)
        self.assertEqual(
            container_id,
            base._get_uuid(body.get('container_ref')),
            body
        )
        self.assertEqual("generic", body.get('type'), body)
        self.assertEqual("empty-container", body.get('name'), body)
        self.assertEqual("ACTIVE", body.get('status'), body)
        self.assertEmpty(body.get('secret_refs'), body)
        self.assertEmpty(body.get('consumers'), body)
        self.assertIn('created', body, body)
        self.assertIn('updated', body, body)
        self.assertIn('creator_id', body, body)

        # Verify that the container can be found via generic listing.
        body = self.container_client.list_containers()
        self.assertEqual(1, body.get('total'), body)
        self.assertEqual(1, len(body.get('containers')), body)

        container = body.get('containers')[0]
        self.assertEqual(
            container_id,
            base._get_uuid(container.get('container_ref')),
            container
        )
        self.assertEqual("generic", container.get('type'), container)
        self.assertEqual("empty-container", container.get('name'), container)
        self.assertEqual("ACTIVE", container.get('status'), container)
        self.assertEmpty(container.get('secret_refs'), container)
        self.assertEmpty(container.get('consumers'), container)
        self.assertIn('created', container, container)
        self.assertIn('updated', container, container)
        self.assertIn('creator_id', container, container)

        # Leave the container behind to get cleaned up by infra.

    @decorators.idempotent_id('af10a78d-b1f8-440d-8b89-639861f16fd0')
    def test_add_to_delete_from_container(self):
        # Create a container to test against.
        body = self.create_container(type="generic", name="test-container")
        container_id = base._get_uuid(body.get('container_ref'))

        # Create some secrets to store in the container
        body = self.create_secret()
        secret1_id = base._get_uuid(body.get('secret_ref'))
        body = self.create_secret()
        secret2_id = base._get_uuid(body.get('secret_ref'))

        # Confirm that the container is empty
        body = self.container_client.get_container(container_id)
        self.assertEqual(
            container_id,
            base._get_uuid(body.get('container_ref')),
            body
        )
        self.assertEmpty(body.get('secret_refs'), body)

        # Add the secrets to the container
        self.container_client.add_secret_to_container(
            container_id,
            secret1_id
        )
        self.container_client.add_secret_to_container(
            container_id,
            secret2_id
        )

        # Confirm that the secrets are in the container
        body = self.container_client.get_container(container_id)
        self.assertEqual(
            container_id,
            base._get_uuid(body.get('container_ref')),
            body
        )
        self.assertEqual(2, len(body.get('secret_refs')), body)
        for secret_ref in body.get('secret_refs'):
            secret_id = base._get_uuid(secret_ref.get('secret_ref'))
            self.assertIn(secret_id, (secret1_id, secret2_id))

        # Remove the secrets from the container
        self.container_client.delete_secret_from_container(
            container_id,
            secret1_id
        )
        self.container_client.delete_secret_from_container(
            container_id,
            secret2_id
        )

        # Confirm that the container is empty
        body = self.container_client.get_container(container_id)
        self.assertEqual(
            container_id,
            base._get_uuid(body.get('container_ref')),
            body
        )
        self.assertEmpty(body.get('secret_refs'), body)

        # Clean up the containe
        self.delete_container(container_id)



# Copyright (c) 2016 SAP SE
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from barbican_tempest_plugin.services.key_manager.json.consumer_client \
    import ConsumerClient
from barbican_tempest_plugin.services.key_manager.json.container_client \
    import ContainerClient
from barbican_tempest_plugin.services.key_manager.json.order_client \
    import OrderClient
from barbican_tempest_plugin.services.key_manager.json.quota_client \
    import QuotaClient
from barbican_tempest_plugin.services.key_manager.json.secret_client \
    import SecretClient
from barbican_tempest_plugin.services.key_manager.json.secret_metadata_client \
    import SecretMetadataClient
from barbican_tempest_plugin.services.key_manager.json.secret_stores_client \
    import SecretStoresClient
from barbican_tempest_plugin.services.key_manager.json.transport_key_client \
    import TransportKeyClient

__all__ = [
    'ConsumerClient',
    'ContainerClient',
    'OrderClient',
    'QuotaClient',
    'SecretClient',
    'SecretMetadataClient',
    'SecretStoresClient',
    'TransportKeyClient'
]
